.custom-table-row {
  display: flex;
  margin: 5px 0;
}
.custom-table-td {
  flex: 1;
  margin: 0 10px;
}
.custom-table-td-last {
  text-align: right;
}
.custom-table-th {
  flex:1;
  margin:0 2px;
  font-weight: bold
}
.event-info-table {
  padding: 10px;
  margin-right: 2px;
  height: 100%;
  border: 1px solid transparent;
  border-radius: 4px;
  border-color: #ddd;
}